<?
/* парсер ini-файлов */
class mcIni
{
  private $sections;	// распарсенный файл
  private $defaults = array (
    'widget-panel.hiddenfiles-sign-show'  => '.',
    'widget-panel.hiddenfiles-sign-hide'  => '.',
    'widget-panel.history-prev-item-sign' => '<',
    'widget-panel.history-next-item-sign' => '>',
    'widget-panel.history-show-list-sign' => '^',
  );

  /* функция чтения файла целиком */
  private function readAllFile($filename)
  {
    return file_get_contents($filename);
  }

  /* парсим значение */
  private function parseValues($keyContent)
  {
    $values = explode(';', $keyContent);
    if (sizeof($values) == 1)
      $values = $values[0];
    return $values;
  }

  /* парсим ключ */
  private function parseKeys($sectionContent)
  {
    $keys = Array();
    $matches = Array();
    preg_match_all('/\s*([a-zA-Z0-9-_]+)\s*=\s*([^\n]+)\s*\n/', $sectionContent, &$matches, PREG_OFFSET_CAPTURE);
    $keys_count = sizeof($matches[1]);
    for($i=0; $i<$keys_count; $i++)
      $keys[$matches[1][$i][0]] = $this->parseValues($matches[2][$i][0]);
    return $keys;
  }

  /* парсим секцию */
  private function parseSections($iniContent)
  {
    $sections = array();
    $matches = array();
    if ($iniContent !== false)
    {
	preg_match_all('/\[([a-zA-Z0-9-]+)\]((\s+[^\n=]+=[^\n]+\n)+)/', $iniContent, &$matches, PREG_OFFSET_CAPTURE);
	$sections_count = sizeof($matches[1]);
	for($i=0; $i<$sections_count; $i++)
	    $sections[$matches[1][$i][0]] = $this->parseKeys($matches[2][$i][0]);
    }
    return $sections;
  }

  /* конструктор */
  function __construct($filename)
  {
    $this->sections = $this->parseSections($this->readAllFile($filename));
  }

  /* получение списка секций */
  function getSections()
  {
    return array_keys($this->sections);
  }

  /* получение списка ключей в секции */
  function getKeys($section)
  {
    if (!empty($this->sections[$section]))
	return array_keys($this->sections[$section]);
    return array();
  }

  /* получение значения ключа (<имя секции>.<ключ>) */
  function getValue($key)
  {
    $matches = Array();
    preg_match_all('/^([^\.]+)\.([^\.]+)$/', $key, &$matches, PREG_OFFSET_CAPTURE);
    if (!empty ($this->sections[$matches[1][0][0]][$matches[2][0][0]]))
	return $this->sections[$matches[1][0][0]][$matches[2][0][0]];
    if (!empty($this->defaults[$key]))
	return $this->defaults[$key];
    return $key;
  }

  /* получение значений всех ключей */
  function getAllValues()
  {
    $ret = array();
    foreach ($this->getSections() as $section)
    {
	foreach ($this->getKeys($section) as $key)
	{
	    $ret[$section.".".$key] = $this->sections[$section][$key];
	}
    }
    return $ret;
  }

  /* как пример использования предыдущей функции - получение имени скина (в формате, пригодном для задания имени класса) */
  function getSkinName()
  {
    return mb_ereg_replace('[^a-zA-Z0-9-]', '_', $this->getValue('skin.description'));
  }
  function getUniqName()
  {
    return sha1($this->getSkinName());
  }
}
?>